/**
 * This class represents the stage, or wall of doors, in the Monty Hall simulation.
 * It takes care of all the drawing.
 *
 * @author Nathaniel Titteton
 * @version 0.2
 */

// These imports are necessary to make the graphics work
import javax.swing.*;

// no voice!
//import com.sun.speech.freetts.FreeTTS;
//import com.sun.speech.freetts.InputMode;
//import com.sun.speech.freetts.Voice;
//import com.sun.speech.freetts.VoiceManager;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;


// Note: this class contains some concepts that haven't been covered (and won't be covered)
// in this course.  We've separted that material and placed it toward the end of this
// file.
public class Display {

   // Set to true if we want to skip the animations, etc.
   // This is used by the Simulator class.
   public boolean makeThingsFast=false;
   
   // no voice!
   // public FreeTTS textToSpeech;

   public Display() {
       // no voice!
       //VoiceManager voiceManager = VoiceManager.getInstance();
       //Voice voice = voiceManager.getVoice("kevin16");

       //textToSpeech = new FreeTTS(voice);
       //textToSpeech.setInputMode(InputMode.TEXT);
       //textToSpeech.startup();
   }

   // called only once to place the graphics, create the images, etc.
   public void buildTheStage() {
       uglySetup();
   }

   // called before each game
   public void setupNewGame() {
       say("Hello.");
       closeAllDoors();
       montyReady();
       playerReady();
       updateGraphics();
   }

   // happens once per game
   public void placeCar(int door) {
       doorWithNumber(door).makeCarDoor();
   }


   public void markInitialChoice(int door) {
       //doorWithNumber(door).makeChecked();
   }


   public void openDoor(int door) {
       doorWithNumber(door).open();
       montyPanel.revealGoat();
       updateGraphics();
   }



   public void say(String message) {
       instructions.setText(message);
       if (!makeThingsFast) {
           // no voice!
           //textToSpeech.textToSpeech(message);
       }
       updateGraphics();
   }



   ///////////////////////////////////////////////////////////////////////////////////
   // Here there be ugliness.
   //
   // Below here are the procedures, variables, and general Java craziness necessary
   // to make the graphics work.  By all means examine the code, but it probably isn't
   // worth your time to understand everything.

   // graphics variables
   private JFrame frame;
   public JLabel instructions;
   private JLabel results;
   private DoorPanel leftDoor;
   private DoorPanel middleDoor;
   private DoorPanel rightDoor;
   private JLabel leftDoorLabel;
   private JPanel leftArea, middleArea, rightArea, topArea, bottomCenterArea;
   private PlayerPanel playerPanel;
   private MontyPanel montyPanel;
   //private
   private JButton leftButton, middleButton, rightButton;
   private JButton stayButton, switchButton;

   // pictures
   // it would be nice if these didn't get loaded each time a display was made....
   private Image carImage = new ImageIcon(getClass().getClassLoader().getResource("car.jpg")).getImage();
   private Image goatImage = new ImageIcon(getClass().getClassLoader().getResource("goat2.jpg")).getImage();
   private Image checkmarkImage = new ImageIcon(getClass().getClassLoader().getResource("checkmark.jpg")).getImage();   
   private Image questionmarkImage = new ImageIcon(getClass().getClassLoader().getResource("questionmark.jpg")).getImage();
   private Image door1Image = new ImageIcon(getClass().getClassLoader().getResource("Door1.jpg")).getImage();
   private Image door2Image = new ImageIcon(getClass().getClassLoader().getResource("Door2.jpg")).getImage();
   private Image door3Image = new ImageIcon(getClass().getClassLoader().getResource("Door3.jpg")).getImage();
   private Image montyIdleImage = new ImageIcon(getClass().getClassLoader().getResource("Monty_idle.jpg")).getImage();
   private Image montyRevealGoatImage = new ImageIcon(getClass().getClassLoader().getResource("Monty_revealGoat.jpg")).getImage();
   private Image montySecondChoiceImage = new ImageIcon(getClass().getClassLoader().getResource("Monty_secondChoice.jpg")).getImage();
   private Image montyWinImage = new ImageIcon(getClass().getClassLoader().getResource("Monty_win.jpg")).getImage();
   private Image montyLoseImage = new ImageIcon(getClass().getClassLoader().getResource("Monty_lose.jpg")).getImage();

   private Image contestantThinkingImage = new ImageIcon(getClass().getClassLoader().getResource("Contestant_thinking.jpg")).getImage();
   private Image contestantChoose1Image = new ImageIcon(getClass().getClassLoader().getResource("Contestant_choose1.jpg")).getImage();
   private Image contestantChoose2Image = new ImageIcon(getClass().getClassLoader().getResource("Contestant_choose2.jpg")).getImage();
   private Image contestantChoose3Image = new ImageIcon(getClass().getClassLoader().getResource("Contestant_choose3.jpg")).getImage();
   private Image contestantWinImage = new ImageIcon(getClass().getClassLoader().getResource("Contestant_win.jpg")).getImage();
   private Image contestantLoseImage = new ImageIcon(getClass().getClassLoader().getResource("Contestant_lose.jpg")).getImage();
   


   private void uglySetup() {
       frame = new JFrame();
       frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
       Container contentPane = frame.getContentPane();

       instructions = new JLabel();
       instructions.setText("Hello. Let's play a game.");
       topArea = new JPanel();
       topArea.add(instructions);
       contentPane.setLayout(new GridBagLayout());
       GridBagConstraints c = new GridBagConstraints();
       c.fill = GridBagConstraints.HORIZONTAL;
       c.gridwidth = 3;
       c.gridx = 0;
       c.gridy = 0;
       contentPane.add(topArea, c);
       
       Dimension dimDoor = new Dimension(280,250);
       Dimension dimMonty = new Dimension(120,200);
       Dimension dimContestant = new Dimension(120,200);

       leftDoor = new DoorPanel(this);
       leftDoor.setImage(door1Image);
       leftDoor.setClosedDoorImage(door1Image);
       leftDoor.setMinimumSize(dimDoor);
       leftDoor.setPreferredSize(dimDoor);
       leftDoor.setToolTipText("Door #1");
       leftArea = new JPanel();
       leftArea.add(leftDoor, BorderLayout.CENTER);
       c.fill = GridBagConstraints.NONE;
       c.gridwidth = 1;
       c.gridx = 0;
       c.gridy = 1;
       contentPane.add(leftArea, c);

       middleDoor = new DoorPanel(this);
       middleDoor.setImage(door2Image);
       middleDoor.setClosedDoorImage(door2Image);
       middleDoor.setMinimumSize(dimDoor);
       middleDoor.setPreferredSize(dimDoor);
       middleDoor.setToolTipText("Door #2");
       middleArea = new JPanel();
       middleArea.add(middleDoor, BorderLayout.CENTER);
       c.gridx = 1;
       c.gridy = 1;
       contentPane.add(middleArea, c);

       rightDoor = new DoorPanel(this);
       rightDoor.setImage(door3Image);
       rightDoor.setClosedDoorImage(door3Image);
       rightDoor.setMinimumSize(dimDoor);
       rightDoor.setPreferredSize(dimDoor);
       rightDoor.setToolTipText("Door #3");
       rightArea = new JPanel();
       rightArea.add(rightDoor, BorderLayout.CENTER);
       c.gridx = 2;
       c.gridy = 1;
       contentPane.add(rightArea, c);

       playerPanel = new PlayerPanel(this);
       playerPanel.setImage(contestantThinkingImage);
       playerPanel.setMinimumSize(dimContestant);
       playerPanel.setPreferredSize(dimContestant);
       playerPanel.setToolTipText("Contestant (You)");
       c.ipadx = 15;
       c.gridx = 0;
       c.gridy = 2;
       contentPane.add(playerPanel, c);

       bottomCenterArea = new JPanel();
       bottomCenterArea.setLayout(new GridBagLayout());
       c.gridx = 1;
       c.gridy = 2;
       contentPane.add(bottomCenterArea, c);

       montyPanel = new MontyPanel(this);
       montyPanel.setImage(montyIdleImage);
       montyPanel.setMinimumSize(dimMonty);
       montyPanel.setPreferredSize(dimMonty);
       montyPanel.setToolTipText("Monty (host)");
       c.ipadx = 15;
       c.gridx = 2;
       c.gridy = 2;
       contentPane.add(montyPanel, c);
       
       frame.setSize(850,600);
       frame.setVisible(true);
   }


   // returns the graphics object for the door with a given number
   private DoorPanel doorWithNumber(int num) {
       if (num == 1)
           return(leftDoor);
       else if (num == 2)
           return(middleDoor);
       else
           return(rightDoor);
       }


   private void updateGraphics() {
       if (!makeThingsFast)
           frame.repaint();
   }


   public void closeAllDoors() {
       leftDoor.close();
       middleDoor.close();
       rightDoor.close();
       updateGraphics();
   }

   public void montyReady() {
       this.montyPanel.getReady();
   }
    
   public void playerReady() {
       this.playerPanel.getReady();
    }

   /**
    * Graphics widget for the doors
    */
   class DoorPanel extends JPanel implements ActionListener {

       Image i;
       Image closedDoorImage;
       Display d;
       boolean open = false;
       boolean carDoor = false;

       public DoorPanel(Display d) {
           this.d = d;
       }

       // use useGraphics here?
       public void paintComponent(Graphics g) {
              g.drawImage(i,0,0,this);                  // draw door image
       }

       public void open() {
           if (carDoor) {
               setImage(d.carImage);
           } else {
               setImage(d.goatImage);
           }
           open = true;
       }

       public void close() {
           // this also resets the door...
           setImage(this.closedDoorImage);
           carDoor = false;
           open = false;
       }

       public void makeCarDoor() {
           carDoor = true;
       }

       public void setImage(Image newImage) {
           i = newImage;
       }
       
       public void setClosedDoorImage(Image closedDoorImage) {
           this.closedDoorImage = closedDoorImage;  
        }

    public void actionPerformed(ActionEvent e) {
        System.out.println("clicked");
    }

   }  // close Door Panel
   
   
   /**
    * Graphics widget for the doors
    */
   class MontyPanel extends JPanel implements ActionListener {

       Image i;
       Display d;

       public MontyPanel(Display d) {
           this.d = d;
       }

       public void revealGoat() {
           this.setImage(montyRevealGoatImage);
       }

    // use useGraphics here?
       public void paintComponent(Graphics g) {
           g.drawImage(i,0,0,this);                  // draw door image
       }

       public void setImage(Image newImage) {
           i = newImage;
       }

    public void actionPerformed(ActionEvent e) {
        System.out.println("clicked");
    }

    public void askSwitchOrStay() {
        this.setImage(montySecondChoiceImage);
    }

    public void celebrate() {
        this.setImage(montyWinImage);
    }

    public void mourn() {
        this.setImage(montyLoseImage);
    }
    
    public void getReady() {
        this.setImage(montyIdleImage);
    }

   }  // close Monty Panel

   /**
    * Graphics widget for the doors
    */
   class PlayerPanel extends JPanel implements ActionListener {

       Image i;
       Display d;

       public PlayerPanel(Display d) {
           this.d = d;
       }

       // use useGraphics here?
       public void paintComponent(Graphics g) {
           g.drawImage(i,0,0,this);                  // draw door image
       }

       public void setImage(Image newImage) {
           i = newImage;
       }

    public void actionPerformed(ActionEvent e) {
        System.out.println("clicked");
    }

    public void chooseInitialDoor(int initialDoorChoice) {
        switch (initialDoorChoice) {
           case 1: setImage(contestantChoose1Image);
                   break;
           case 2: setImage(contestantChoose2Image);
                   break;
           case 3: setImage(contestantChoose3Image);
                   break;
        }
    }

    public void celebrate() {
        this.setImage(contestantWinImage);
    }

    public void mourn() {
        this.setImage(contestantLoseImage);
    }
    
    public void getReady() {
        this.setImage(contestantThinkingImage);
       }

   }  // close Monty Panel


   public void setDoorsSelectableByPlayer(final Player player) {
       this.leftButton = new JButton("door #1");
       this.leftButton.setToolTipText("Click to choose door 1");
       this.leftButton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent arg0) {
            player.initialDoorChoice = 1;
        }
       });
       this.bottomCenterArea.add(this.leftButton);

       this.middleButton = new JButton("door #2");
       this.middleButton.setToolTipText("Click to choose door 2");
       this.middleButton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent arg0) {
            player.initialDoorChoice = 2;
        }
       });
       this.bottomCenterArea.add(this.middleButton);

       this.rightButton = new JButton("door #3");
       this.rightButton.setToolTipText("Click to choose door 3");
       this.rightButton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent arg0) {
            player.initialDoorChoice = 3;
        }
       });
       this.bottomCenterArea.add(this.rightButton);

       updateGraphics();
    }

   public void setDoorsUnselectable() {
       //this.bottomCenterArea.remove(this.leftButton);
       //this.bottomCenterArea.remove(this.middleButton);
       //this.bottomCenterArea.remove(this.rightButton);
       this.bottomCenterArea.removeAll();

       updateGraphics();
   }

   public void displaySwitchOrStayOptionsForPlayer(final Player thePlayer) {
       this.stayButton = new JButton("stay");
       this.stayButton.setToolTipText("Click to stay");
       this.stayButton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent arg0) {
            thePlayer.switchOrStay = "stay";
            stayButton.setEnabled(false);
            switchButton.setEnabled(false);
        }
       });
       this.switchButton = new JButton("switch");
       this.switchButton.setToolTipText("Click to switch to another door");
       this.switchButton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent arg0) {
            thePlayer.switchOrStay = "switch";
            stayButton.setEnabled(false);
            switchButton.setEnabled(false);
        }
       });
       this.bottomCenterArea.add(stayButton);
       this.bottomCenterArea.add(switchButton);
       
       this.montyPanel.askSwitchOrStay();
       updateGraphics();
   }

   public void hideSwitchOrStayOptions() {
       //this.bottomCenterArea.remove(stayButton);
       //this.bottomCenterArea.remove(switchButton);
       this.bottomCenterArea.removeAll();
       updateGraphics();
   }

   public void chooseInitialDoor(int initialDoorChoice) {
       this.playerPanel.chooseInitialDoor(initialDoorChoice);
}

   public void celebrate() {
       this.playerPanel.celebrate();
       this.montyPanel.celebrate();
   }

   public void mourn() {
       this.playerPanel.mourn();
       this.montyPanel.mourn();
   }
   
} // close Display