/**
 * This class represents the player, or contestant, in the Monty Hall simulation.
 *
 * @author Nathaniel Titterton
 * @version 0.2
 */
public class Player
{

   // if this is true, then this player isn't thinking for themselves, but rather
   // taking orders from some other object.  (This will let us 'program' the
   // Player...)
   public boolean takeOrders = false;
   // if taking orders, this determines who to take orders from.
   public Simulator myController;

   // starts with 0, ends with 1,2,or3
   public int initialDoorChoice;
   public String switchOrStay = "";
   
   public Display theDisplay;
   
   public Player(Display display) {
       theDisplay = display;
    }

   // get the user to choose the initial door, or get it from the controller
   public int chooseInitialDoor() {
       if (takeOrders) {
           return(myController.chooseInitialDoorStrategy());
       } else {
    	   if (initialDoorChoice > 0 && initialDoorChoice < 4) {
    	     theDisplay.chooseInitialDoor(initialDoorChoice);
    	   }
		return this.initialDoorChoice;
       }
   }


   // get whether to swtich or not from the user, or take orders
   // is it 'important' to get the parameters here?
   public String switchOrStay (int initialDoor, int revealedDoor) {
       if (takeOrders) {
           return(myController.switchOrStayStrategy(initialDoor, revealedDoor));
       } else {
    	   return this.switchOrStay;
       }
    }


   public void setDisplay(Display display) {
	   this.theDisplay = display;
	   this.initialDoorChoice = 0;
	   this.switchOrStay = "";
   }
}